/*
 * pilot.c
 * Copyright (C) 2014-2019 Peter Belkner <pbelkner@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <bg_pilot.h>

#define PILOT_CALLBACK
#if defined (PILOT_CALLBACK) // [
static int pilot_callback_leaf_enter(bg_pilot_hist_t *hist, void *data)
{
  unsigned *count=data;

  ++*count;
#if defined (_WIN32) // [
  fwprintf(stdout,L"%u: \"%s\"\n",*count,hist->path);
#else // ] [
  fprintf(stdout,"%u: \"%s\"\n",*count,hist->path);
#endif // ]
  fflush(stdout);

  return 0;
}

static void pilot_callback_leaf_leave(bg_pilot_hist_t *hist FFUNUSED,
    void *data FFUNUSED)
{
}

static int pilot_callback_branch_enter(bg_pilot_hist_t *hist FFUNUSED,
    void *data FFUNUSED)
{
  return 0;
}

static void pilot_callback_branch_leave(bg_pilot_hist_t *hist FFUNUSED,
    void *data FFUNUSED)
{
}

static bg_pilot_callback_t pilot_callback={
  .leaf.enter=pilot_callback_leaf_enter,
  .leaf.leave=pilot_callback_leaf_leave,
  .branch.enter=pilot_callback_branch_enter,
  .branch.leave=pilot_callback_branch_leave,
};
#endif // ]

#if defined (_WIN32) // [
int wmain(int argc, wchar_t *const *argv)
#else // ] [
int main(int argc, char *const *argv)
#endif // ]
{
  unsigned count=0u;
  bg_pilot_t pilot;
  int i;

#if defined (PILOT_CALLBACK) // [
  if (bg_pilot_create(&pilot,20,&pilot_callback,&count)<0) {
    DMESSAGE("creating pilot");
    goto e_pilot;
  }
#else // ] [
  if (bg_pilot_create(&pilot,20,NULL,NULL)<0) {
    DMESSAGE("creating pilot");
    goto e_pilot;
  }
#endif // ]

  for (i=1;i<argc;++i) {
#if defined (PILOT_CALLBACK) // [
    bg_pilot_loop(&pilot,argv[i]);
#else // ] [
    bg_pilot_first(&pilot,argv[i]);

    while (!bg_pilot_empty(&pilot)) {
      ++count;
#if defined (_WIN32) // [
      fwprintf(stdout,L"%u: \"%s\"\n",count,(pilot.nxt-1)->path);
#else // ] [
      fprintf(stdout,"%u: \"%s\"\n",count,(pilot.nxt-1)->path);
#endif // ]
      fflush(stdout);
      bg_pilot_next(&pilot,0);
    }
#endif // ]
  }

  bg_pilot_destroy(&pilot);
e_pilot:
  return 0;
}
